#include <avr/io.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>
#include <avr/interrupt.h>
#include "utility.h"
#include "init.h"	

#define AMBIENT_LIGHT 0
#define UP 1
#define LEFT 1
#define DOWN 0
#define RIGHT 0

unsigned char IRtoDist(unsigned int IR);
unsigned char DisttoServo(unsigned char distance);

volatile bool readIRavg = false, readIR = false, setServoFlag = true, downCounting = false, neighborFlag = false, first = true;
volatile unsigned int IRdist = 0, up=0,left=0,down=0,right=0; 
volatile unsigned int IRacc = 0, servoAvg[10];
volatile unsigned char IRcounter = 0;
volatile unsigned char neighbor = 0;


unsigned int distanceArray[7] = {1010, 820, 760, 740, 717, 706, 696};

int main(void)
{	

	cli();
	init_ports();
	init_signal();		
	init_timers();
	init_ADC();
	sei();
	setIR(8);

	unsigned int avg_neighbors = 0;
	unsigned int servo = 0;

	while(1)
	{
		if (setServoFlag)
		{
			//Average servo values
			for(unsigned char cnt=0; cnt<9; cnt++)
				servoAvg[cnt] = servoAvg[cnt+1];
			if(AMBIENT_LIGHT)
			{
				servoAvg[9] = (unsigned char)(IRacc>>5)*2;
				IRcounter = 0;
			}
			else
			{
				servoAvg[9] = DisttoServo(IRtoDist(IRdist));
			}
			for(unsigned char cnt=0; cnt<10;cnt++)
				servo += servoAvg[cnt];
			servo = servo/10;

			//Avg of neighbors
			avg_neighbors = (up+left+down+right)/(UP+LEFT+DOWN+RIGHT)>>6; //Scale down to fit servo-output

			//Output
			setServo(((unsigned char)(servo)*0.7 + avg_neighbors*0.3));

			setServoFlag = false;
		}

	}

	return 0;

}


unsigned char IRtoDist(unsigned int IR)
{
	for(unsigned char i=0; i<= 6; i++)
	{
		if(IR > distanceArray[i])
			return i;
	}
	return 7;
/*  //Worked well in the first version, but for some reason not in the second?!
	double distance = 582150*pow(((double)IR),-8.688);
	if(distance > 33)
		distance = 33;
	return (unsigned char)distance;
*/
}

unsigned char DisttoServo(unsigned char distance)
{
	unsigned char servo = distance*5+2;
	if(servo < 2)		//Collapsed
		servo = 2;
	else if(servo > 35)	//Expanded
		servo = 35;
	return servo;
}

ISR(ADC_vect)
{
	unsigned int low = 0, high = 0; 
	low = ADCL; 
	high = ADCH;	

	if(readIRavg)
	{
		if(IRcounter < 35)
		{
			ADCSRA |= 0b01000000; // Start ADC				
			if(IRcounter == 0)
			{
				IRdist = (high<<8)+low - (IRacc>>5);
				if(IRdist > 810)
					setLED(1);
				else
					setLED(0);
				IRacc = 0;
				setServoFlag = true;
			}
			else if(IRcounter > 3)
			{
				IRacc += (high<<8)+low;
			}
			IRcounter++;
		}
		else
			readIRavg = false;
	}
}

SIGNAL(PCINT0_vect)
{
	if(first)
	{
		//start timer
		TCNT1 = 0;
		first = false;
	}
	else
	{
		unsigned char time = TCNT1;
		if(time > 3000)		//counting the wrong flank
			TCNT1 = 0;
		else if(neighbor == 0) //Up
			up = time;			
		else if(neighbor == 3) //Right
			right = time;
		first = true;
		neighbor++;
		if(neighbor > 3) neighbor = 0;
		while((neighbor == 0 && !UP) || (neighbor == 1 && !LEFT) || (neighbor == 2 && !DOWN) || (neighbor == 3 && !RIGHT))
			neighbor++;	//make sure sensors exist
	}
}

SIGNAL(PCINT1_vect)
{
	if(first)
	{
		//start timer
		TCNT1 = 0;
		first = false;
	}
	else
	{
		unsigned char time = TCNT1;
		if(time > 3000) 		//counting the wrong flank
			TCNT1 = 0;
		else if(neighbor == 1) //Left
			left = time;			
		else if(neighbor == 2) //down
			down = time;
		first = true;
		neighbor++;
		if(neighbor > 3) neighbor = 0;
		while((neighbor == 0 && !UP) || (neighbor == 1 && !LEFT) || (neighbor == 2 && !DOWN) || (neighbor == 3 && !RIGHT))
			neighbor++;	//make sure sensors exist
	}
}

SIGNAL(TIM0_OVF_vect) 
{	
	readIRavg = true;
	ADCSRA |= 0b01000000; // Start ADC
	IRcounter = 0;
}

SIGNAL(TIM0_COMPB_vect)
{
	if(downCounting)
	{
		downCounting = false; 
	}
	else
	{
		downCounting = true;
	}
	//For old version, technically not needed right now
}

